;**************************************************************************
;* FileName	: I2C_HW.INC
;* Date 	: 09.10.2002
;* File version : v1.01
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: Hardware I2C functions
;*		  ==================================
;*		  I2C_startcond
;*		  I2C_restartcond
;*		  I2C_stopcond
;*		  I2C_send
;*		  I2C_receive
;*		  I2C_ack
;*
;*
;*-------------------------------------------------------------------------
;* Needed defines in main-modul:
;* 
;* xtal_freq	EQU	4000000	  	; crystal frequency
;* core_freq	EQU	xtal_freq/4	; core operating frequency
;*
;* #define		I2C_SCL		PORTC,3		; out
;* #define		I2C_SDA		PORTC,4		; in/out
;*
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;* #define DEBUG			; for some debug message
;* #define NO_I2C			; for faster simulation
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  02/10/2000 	Initial Release
;* 1.01   09/10/2002    reformatting source
;*
;**************************************************************************
	ifdef INC_I2C


;***** I2C_startcond ******************************************************
;* Generate a start conditions
;* Input : -				; 
;* Output: -				;
;* Clock : ca. 2/Fc			; Fc = speed of I2C bus
;**************************************************************************
I2C_startcond	bank1
		ERRORLEVEL -302
		bsf	SSPCON2, SEN
		ERRORLEVEL +302
		bank0		
I2C_sc1		btfss	PIR1, SSPIF
		goto	I2C_sc1		; wait for end of start condition
		bcf	PIR1, SSPIF
		return


;***** I2C_restartcond ****************************************************
;* Generate a restart conditions
;* Input : -				; 
;* Output: -				;
;* Clock : ca. 3/Fc		
;**************************************************************************
I2C_restartcond	bank1
		ERRORLEVEL -302
		bsf	SSPCON2, RSEN
		ERRORLEVEL +302
		bank0		
I2C_rsc1	btfss	PIR1, SSPIF
		goto	I2C_rsc1	; wait for end of repeated start cond.
		bcf	PIR1, SSPIF
		return


;***** I2C_stopcond *******************************************************
;* Generate a stop conditions
;* Input : -				; 
;* Output: -				;
;* Clock : ca. 3/Fc		
;**************************************************************************
I2C_stopcond	bank1
		ERRORLEVEL -302
		bsf	SSPCON2, PEN
		ERRORLEVEL +302
		bank0		
I2C_pc1		btfss	PIR1, SSPIF
		goto	I2C_pc1		; wait for end of stop condition
		bcf	PIR1, SSPIF
		return


;***** I2C_send ***********************************************************
;* Send a byte via I2C (set Z if not acknowledged)
;* Input : w <- data			; 
;* Output: Z=1 if not acknowledged	;
;* Clock : ca. 8/Fc		
;**************************************************************************
I2C_send	movwf	SSPBUF

I2C_send1	btfss	PIR1, SSPIF
		goto	I2C_send1		; wait for end of transmission
		bcf	PIR1, SSPIF

		bank1
		ERRORLEVEL -302
		bsf	STATUS,Z		
		btfss	SSPCON2, ACKSTAT	; check -ACK bit
		bcf	STATUS,Z
		ERRORLEVEL +302
		bank0
		return



;***** I2C_receive ********************************************************
;* Receive a byte from I2C bus
;* Input : -				; 
;* Output: data -> w			; received data
;* Clock : ca. 8/Fc		
;**************************************************************************
I2C_receive	bank1
		ERRORLEVEL -302
		bsf	SSPCON2, RCEN		; configured as a receiver
		ERRORLEVEL +302
		bank0

I2C_rec1	btfss	PIR1, SSPIF
		goto	I2C_rec1		; wait for data
		bcf	PIR1, SSPIF
		
		movf	SSPBUF ,w		; get data
		return



;***** I2C_ack ************************************************************
;* Acknoledge a transfer
;* Input : C <- ack 			; 0 - no ack, 1 - ack
;* Output: -				;
;* Clock :
;**************************************************************************
I2C_ack		bank1
		ERRORLEVEL -302
		bsf	SSPCON2, ACKDT
		btfsc	STATUS, C
		bcf	SSPCON2, ACKDT		; set acknowledge bit
		bsf	SSPCON2, ACKEN		; start ack. sequence
		ERRORLEVEL +302
		bank0

I2C_rec2	btfss	PIR1, SSPIF
		goto	I2C_rec2		; wait for end of ack.
		bcf	PIR1, SSPIF
		return



;**************************************************************************
	endif
